{% load static reactor %}
<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>Template • TodoMVC</title>
    <!-- CSS overrides - remove if you don't need it -->
    <link rel="stylesheet" href="{% static "todo.css" %}">
    {% reactor_header %}
  </head>
  <body>

      {% component 'x-todo-list' %}

      <!-- This footer should hidden by default and shown when there are todos -->

    </section>
    <footer class="info">
      <p>Double-click to edit a todo</p>
      <!-- Remove the below line ↓ -->
      <p>Template by <a href="http://sindresorhus.com">Sindre Sorhus</a></p>
      <!-- Change this out with your name and url ↓ -->
      <p>Created by <a href="http://todomvc.com">you</a></p>
      <p>Part of <a href="http://todomvc.com">TodoMVC</a></p>
    </footer>
    <!-- Scripts here. Don't remove ↓ -->
  </body>
</html>
