# Generated by Django 2.2 on 2019-04-17 21:27

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Item',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('completed', models.BooleanField(default=False)),
                ('text', models.CharField(max_length=256)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
